'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var EventEmitter = _interopDefault(require('events'));

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

const name = "peek42";
const version = "5.5.0";

function _isBasicObject(val) {
  return typeof val === 'object' && val !== null && val.__proto__ === undefined;
}

function _string(val) {
  return _isBasicObject(val) ? '[object BasicObject]' : String(val);
}

function _comment(comment, val, note = undefined) {
  if (comment !== undefined && comment !== '') {
    return comment;
  }

  let str = _string(val).replace(/\s+/gm, ' ');

  let max = 69;

  if (str.length > max) {
    str = `${str.substr(0, max)}...`;
  }

  return note === undefined ? str : `(${note}) ${str}`;
}

const _outputOptsDefaults = {
  level: 'log',
  collapsed: false
};

function _prettyMakesSense(val) {
  return val instanceof Object && !(val instanceof Function) || _isBasicObject(val);
}

function pretty(val) {
  let objs = [];
  let keys = [];
  return JSON.stringify(val, (k, v) => {
    if (v instanceof Object) {
      let seen = objs.indexOf(v);

      if (seen === -1) {
        objs.push(v);
        keys.push(k || 'ROOT');
        return v;
      }

      return `${v} (ref to ${keys[seen]})`;
    }

    return v;
  }, 2);
}

function p(val, comment = undefined, opts = undefined) {
  peek42._output(val, _comment(comment, val, 'value'), opts);
}

function pp(val, comment = undefined, opts = undefined) {
  peek42._output(_prettyMakesSense(val) ? pretty(val) : val, _comment(comment, val, 'pretty'), opts);
}

function use(lib) {
  Object.assign(p, lib.peek42(p, _comment));
  return peek42;
}

const peek42 = {
  get [Symbol.toStringTag]() {
    return name;
  },

  version: version,
  pretty,
  p,
  pp,
  use
};

const cable = {
  __proto__: EventEmitter.prototype,

  init(websocket) {
    this.emit('ready', websocket);
  },

  get websocket() {
    return new Promise((resolve, reject) => {
      if (this._websocket) {
        resolve(this._websocket);
      } else {
        this.on('ready', websocket => {
          resolve(this._websocket = this._websocket || websocket);
        });
      }
    });
  }

};

const cp = async (msg, comment, opts) => (await cable.websocket).send(JSON.stringify({
  msg,
  comment,
  opts
}));

function _output(val, comment, opts = {}) {
  opts = _objectSpread({}, _outputOptsDefaults, opts);
  let str = comment === null ? _string(val) : `// ${String(comment)}\n${_string(val)}`;
  console[opts.level](str);
}

Object.assign(peek42, {
  _output,
  cable,
  cp
});

module.exports = peek42;
//# sourceMappingURL=peek42.node.js.map
